"""Implement a custom function that is triggered by
pressing a keyboard button when the rendering window
is in interactive mode.
Place the pointer anywhere on the mesh and press c"""
from vedo import dataurl, printc, Plotter, Point, Mesh

#############################################################
def myfnc(evt):
    mesh = evt.object
    # printc('dump event info', evt)
    if not mesh or evt.keypress != "c":
        printc("click mesh and press c", c="r", invert=True)
        return
    printc("point:", mesh.picked3d, c="v")
    cpt = Point(mesh.picked3d)
    cpt.color("violet").ps(20).pickable(False)
    plt.add(cpt).render()

##############################################################
plt = Plotter(axes=1)
plt+= Mesh(dataurl+"bunny.obj").color("gold")
plt+= __doc__
plt.add_callback('on key press', myfnc)
plt.show().close()



"""Insert 2D and 3D scalarbars in the rendering scene"""
from vedo import Mesh, dataurl, show

shape = Mesh(dataurl + "lamp.vtk")

ms = []
cmaps = ("jet", "PuOr", "viridis")
for i in range(3):
    s = shape.clone(deep=False).pos([0, i * 2.2, 0])
    # colorize mesh
    scalars = s.points[:, 2]
    s.cmap(cmaps[i], scalars)
    ms.append(s)

# add 2D scalar bar to first mesh
ms[0].add_scalarbar(title="my scalarbar\nnumber #0")  # 2D

# add 3D scalar bars
ms[1].add_scalarbar3d(c="k", title="scalarbar #1", size=[0, 3])

sc = ms[2].add_scalarbar3d(
    c="k",
    size=[None, 2.8],  # change y-size only
    title="A viridis colormap\nscalarbar to play with",
    title_font="Quikhand",
    title_xoffset=-2,  # offset of labels
    title_size=1.5,
)
sc.scalarbar.rotate_x(90).scale(1.2).shift(0,2,0)  # make it vertical

# create a 2D copy scalarbar to the 3D one
sc2d = sc.scalarbar.clone2d(size=0.3, ontop=True)

show(ms, sc2d, __doc__, axes=1, viewup="z").close()



"""Hover mouse onto an object
to pop a flag-style label"""
from vedo import *

b = Mesh(dataurl + "bunny.obj")
b.color("purple5").legend("Bugs the bunny")
c = Cube(side=0.1).y(-0.02).compute_normals()
c.alpha(0.8).lighting("off").lw(1).legend("The Cube box")

cap = c.caption(
    "2d caption for a cube\nwith face indices",
    point=[0.044, 0.03, -0.04],
    size=(0.3, 0.06),
    font="VictorMono",
    alpha=1,
)

# create a new object made of polygonal text labels to indicate the cell numbers
flabs = c.labels("id", on="cells", font="Theemim", scale=0.02, c="k")
vlabs = c.clone().clean().labels2d(font="ComicMono", scale=3, bc="orange7")

# create a custom entry to the legend
lbox = LegendBox([b, c], font="Bongas", width=0.25, bg='blue6')

with Plotter(axes=11, bg2="linen") as plt:
    plt.add(b, c, cap, flabs, vlabs, lbox, __doc__)
    plt.add_hint(b, "My fave bunny")
    plt.show()



"""Nelder-Mead minimization algorithm for the 4D function:
F = (x/4-1)**2 + (y+2)**2 + z**2 + (w-6)**2 + 3"""
from vedo import Minimizer, Line, show
from vedo.pyplot import plot

def func(pars):
    x, y, z, w = pars  # unpack parameters for convenience
    F = (x/4-1)**2 + (y+2)**2 + z**2 + (w-6)**2 + 3
    return F

mini = Minimizer(func)
mini.set_parameter("x", 4.0) # set initial values
mini.set_parameter("y", -3.0)
mini.set_parameter("z", 1.0)
mini.set_parameter("w", 1.0)
res = mini.minimize()  # run the minimization
mini.compute_hessian() # compute the Hessian to estimate the errors
print(mini)

# Draw the path of the minimization
path = res["parameters_path"]
vals = res["function_path"]
line = Line(path[:,:3], lw=5).cmap("jet", path[:,3]).add_scalarbar()
plo = plot(vals, xtitle="iteration", ytitle="function eval", lw=3)
show(line, plo.clone2d(), __doc__, axes=1)




from vedo import *

settings.default_font = "VictorMono"

r = 0.2

points_2d = np.random.rand(10000, 2) - 0.5
values = np.random.rand(10000)
vmin, vmax = values.min(), values.max()

pcloud1 = Points(points_2d, c='k', r=5).rotate_x(30)
pcloud2 = pcloud1.clone().cut_with_cylinder(r=r, invert=True)

cyl = Cylinder(r=r, height=1, res=360).alpha(0.2)
dists = pcloud1.distance_to(cyl, signed=True)
mask = dists < 0
print("The boolean mask is", mask)

pcloud1.pointdata['values'] = values
pcloud1.pointdata['MASK'] = mask

pts1 = pcloud1.clone().point_size(5)
pts1.cut_with_scalar(0.5, 'MASK')

pts1.cmap('bwr', 'values', vmin=vmin, vmax=vmax).add_scalarbar3d(title='values')
# pts1.cmap('RdYlBu', 'MASK').add_scalarbar3d(title='MASK')
pts1.scalarbar.rotate_x(90)

grid = Grid(res=[100,100]).rotate_x(30)
grid.interpolate_data_from(pcloud1, n=3)
grid.cut_with_cylinder(r=r, invert=True)
grid.cmap('bwr', 'values', vmin=vmin, vmax=vmax).wireframe(False).lw(0)
grid.add_scalarbar3d(title='interpolated values').scalarbar.rotate_x(90)

show(cyl, grid, axes=True)
